@@ -37,16 +37,27 @@ class MoodleService:
         if log_responses:
             self.log_responses_to = str(Path(storage_path) / 'responses.log')
 
-    def interactively_acquire_token(self, use_stored_url: bool = False) -> str:
+    def interactively_acquire_token(
+        self, use_stored_url: bool = False, username: str = None, password: str = None
+    ) -> str:
         """
         Walks the user through executing a login into the Moodle-System to get
         the Token and saves it.
         @return: The Token for Moodle.
         """
-        print('[The following Credentials are not saved, it is only used temporarily to generate a login token.]')
+
+        automated = False
+        stop_automatic_generation = False
+        if username is not None and password is not None:
+            automated = True
+
+        if not automated:
+            print('[The following Credentials are not saved, it is only used temporarily to generate a login token.]')
 
         moodle_token = None
         while moodle_token is None:
+            if stop_automatic_generation and automated:
+                break
 
             if not use_stored_url:
                 moodle_url = input('URL of Moodle:   ')
@@ -63,8 +74,16 @@ class MoodleService:
                 moodle_domain = self.config_helper.get_moodle_domain()
                 moodle_path = self.config_helper.get_moodle_path()
 
-            moodle_username = input('Username for Moodle:   ')
-            moodle_password = getpass('Password for Moodle [no output]:   ')
+            if username is not None:
+                moodle_username = username
+                stop_automatic_generation = True
+            else:
+                moodle_username = input('Username for Moodle:   ')
+
+            if password is not None:
+                moodle_password = password
+            else:
+                moodle_password = getpass('Password for Moodle [no output]:   ')
 
             try:
                 moodle_token, moodle_privatetoken = login_helper.obtain_login_token(
@@ -76,6 +95,9 @@ class MoodleService:
             except (ValueError, RuntimeError) as error:
                 print('Error while communicating with the Moodle System! (%s) Please try again.' % (error))
 
+        if automated is True and moodle_token is None:
+            exit(1)
+
         # Saves the created token and the successful Moodle parameters.
         self.config_helper.set_property('token', moodle_token)
         if moodle_privatetoken is not None:
